package b4a.SM.Locale;

import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;

import android.content.Context;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.os.Build;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Formatter;
import java.util.Locale;
import android.os.LocaleList;
import android.view.View;

/**
    * This class is used to change your activity locale.
    */

@BA.Version (1.02f)
@BA.ShortName ("smLocale")
@BA.ActivityObject

public class smLocale
    {
    private static Locale        localeFormatter = null;
    private static NumberFormat  numberFormat    = null;
    private static DecimalFormat decimalFormat   = null;

/**
    * Returns a three-letter abbreviation of activity locale's language.
    */
    public static String getCurrentISO3Language (BA ba)
        {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) { return ba.context.getResources ().getConfiguration ().getLocales ().getDefault ().get (0).getISO3Language (); }
        return Locale.getDefault ().getISO3Language ();
        }

/**
    * Returns a three-letter abbreviation for activity locale's country.
    */
    public static String getCurrentISO3Country (BA ba)
        {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) { return ba.context.getResources ().getConfiguration ().getLocales ().getDefault ().get (0).getISO3Country (); }
        return Locale.getDefault ().getISO3Country ();
        }

/**
    * Returns a name for the activity locale.
    */
    public static String getCurrentDisplayLanguage (BA ba)
        {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) { return ba.context.getResources ().getConfiguration ().getLocales ().getDefault ().get (0).getDisplayLanguage (); }
        return Locale.getDefault ().getDisplayLanguage ();
        }

/**
    * Returns a name for the locale specified by the input parameters.
    */
    public static String getDisplayLanguage (BA ba, String language, String country)
        {
        Locale locale = new Locale (language, country);
        return locale.getDisplayLanguage (new Locale (getCurrentISO3Language (ba), getCurrentISO3Country (ba)));
        }

/**
    * Sets the activity locale using ISO3 codes.
    * Usually recreateActivity = True. But when used in Activity_Create (before loading or creating a layout), you can set recreateActivity = False.
    */
    @SuppressWarnings ("deprecation")
    public static void setLocale (BA ba, String language, String country, Boolean recreateActivity)
        {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) { setRTLDirection (ba, language, country); }
        if ((getCurrentISO3Language (ba) == language) && (getCurrentISO3Country (ba) == country)) { return; }
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N)
            {
            Configuration configuration = ba.context.getResources ().getConfiguration ();
            Locale locale = new Locale (language, country);
            configuration.setLocale (locale);
            LocaleList localeList = new LocaleList (locale);
            LocaleList.setDefault (localeList);
            configuration.setLocales (localeList);
            configuration.setLayoutDirection (locale);
            ba.context.createConfigurationContext (configuration);
            setRTLDirection (ba, language, country);
            }
        else
            {
            Locale locale = new Locale (language, country);
            Locale.setDefault (locale);
            Resources resources = ba.context.getResources ();
            Configuration configuration = resources.getConfiguration ();
            configuration.locale = locale;
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR1) { configuration.setLayoutDirection (locale); }
            resources.updateConfiguration (configuration, resources.getDisplayMetrics ());
            }
        if (recreateActivity) { ba.activity.recreate (); }
        createFormatters (language, country);
        return;
        }

    private static void setRTLDirection (BA ba, String language, String country)
        {
        int direction;
        switch (getCurrentISO3Language (ba))
            {
            case "ara":
                direction = View.LAYOUT_DIRECTION_RTL;
                break;
            case "kur":
                direction = View.LAYOUT_DIRECTION_RTL;
                break;
            case "fas":
                direction = View.LAYOUT_DIRECTION_RTL;
                break;
            case "urd":
                direction = View.LAYOUT_DIRECTION_RTL;
                break;
            case "heb":
                direction = View.LAYOUT_DIRECTION_RTL;
                break;
            case "yid":
                direction = View.LAYOUT_DIRECTION_RTL;
                break;
            default:
                direction = View.LAYOUT_DIRECTION_LTR;
                break;
            }
        ba.activity.getWindow ().getDecorView ().setLayoutDirection (direction);
        }

/**
    * Sets the Formatter locale using ISO3 codes.
    */
    public static void setFormatterLocale (BA ba, String language, String country)
        {
        createFormatters (language, country);
        }

/**
    * Returns a formatted string using the specified locale, format string, and arguments.
    * See https://developer.android.com/reference/java/util/Formatter
    */
    public static String format (BA ba, String stringFormat, Object... arguments)
        {
        if (localeFormatter == null)
           {
           createFormatters (getCurrentISO3Language (ba), getCurrentISO3Country (ba));
           }
        Formatter formatter = new Formatter ();
        try
            {
            return formatter.format (localeFormatter, stringFormat, arguments).toString ();
            }
        catch (Exception e)
            {
            return "";
            }
        }

/**
    * Returns a formatted string using the specified locale, pattern string, and number.
    * See https://developer.android.com/reference/java/text/DecimalFormat
    */
    public static String decimalFormat (BA ba, String stringPattern, Object number)
        {
        if (localeFormatter == null)
           {
           createFormatters (getCurrentISO3Language (ba), getCurrentISO3Country (ba));
           }
        try
            {
            decimalFormat.applyPattern (stringPattern);
            return decimalFormat.format (number);
            }
        catch (Exception e)
            {
            return "";
            }
        }

    private static void createFormatters (String language, String country)
        {
        localeFormatter = new Locale (language, country);
        numberFormat = NumberFormat.getNumberInstance (localeFormatter);
        decimalFormat = (DecimalFormat) numberFormat;
        }

    }
